/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.geoviewer.jxmap.map;

import java.beans.PropertyChangeListener;
import javax.swing.Action;
import nl.cloudfarming.client.geoviewer.Layer;
import nl.cloudfarming.client.geoviewer.render.DrawingRenderer;

/**
 *
 * @author johan
 */
public interface DrawingContext {
    String PROP_DRAWING_CONTEXT = "drawingContext";

    Layer getLayer();
    
    /**
     * Check if the points drawn so far can be used to complete the action.
     * e.g. line not self-crossing; a helpline must split another polygon, ...
     * 
     * @return 
     */
    boolean canFinish(); // 

    /**
     * Finish the drawing action.
     * e.g save a new geometry.
     * 
     * @return true if completing the action succeeded
     */
    boolean finish();

    void cancel();

    DrawingRenderer getRenderer();

    /**
     * Create toolbar actions.
     * 
     * @return 
     */
    Action[] getActions();
    
    void addPropertyChangeListener(PropertyChangeListener listener);
    void removePropertyChangeListener(PropertyChangeListener listener);
    
    String getDescription();
    
}
