/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package nl.cloudfarming.client.geoviewer.jxmap.map.cache;

import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import nl.cloudfarming.client.geoviewer.jxmap.map.cache.spi.DefaultTileService;
import org.openide.util.Lookup;

/**
 * Uses lookup to find all implementations of {@link TileService} interface.
 * 
 * <p>
 * {@link DefaultTileService} is added at the top of the list.
 * </p>
 * 
 * @author Frantisek Post
 */
public class TileRepository {

    private static final TileRepository instance = new TileRepository();
    private List<TileService> services;
    
    private TileRepository() {
        services = new ArrayList<>();
        services.add(DefaultTileService.getInstance()); //default cache first
        services.addAll(findImplementations());
    }
    
    private List<TileService> findImplementations() {
        Collection<? extends TileService> caches = Lookup.getDefault().lookupAll(TileService.class);
        return new ArrayList<>(caches);
    }
    
    private TileService getCacheThatContainsTile(TileCacheInfo tileCacheInfo) {
        TileService tileService = null;
        for (TileService service : services) {
            if (service.contains(tileCacheInfo)) {
                tileService = service;
                break;
            }
        }
        return tileService;
    }
    
    //API
    
    public BufferedImage getTile(TileCacheInfo tileCacheInfo) {
        TileService tileService = getCacheThatContainsTile(tileCacheInfo);
        return (tileService != null) ? tileService.getTile(tileCacheInfo) : null;
    } 
    
    public void writeTile(TileCacheInfo tileCacheInfo, BufferedImage image) {
        DefaultTileService.getInstance().writeTile(tileCacheInfo, image);
    }
    
    public static TileRepository getInstance() {
        return instance;
    }
}
